from mongoengine import *
from lisa.server.web.weblisa.settings import DBNAME
connect(DBNAME)

class Description(EmbeddedDocument):
    lang = StringField(max_length=2)
    description = StringField()

class Plugin(DynamicDocument):
    name = StringField(max_length=120, required=True, help_text='Name of the plugin')
    lang = ListField(StringField(max_length=2), help_text="List of supported languages : ['all','WebSocket']")
    enabled = BooleanField(help_text="Boolean to know if the plugin is enabled or not")
    version = StringField(help_text="The version number of the plugin")
    description = ListField(EmbeddedDocumentField(Description), help_text="Contains a description of the plugin")
    configuration = DictField(help_text="Configuration dictionnary of the plugin")
    meta = {
        'collection': 'plugins',
        'allow_inheritance': False
    }

class Intent(DynamicDocument):
    plugin = ReferenceField(Plugin, reverse_delete_rule=CASCADE)
    name = StringField(required=True, help_text="Name of the intent (whitespaces are _ ). Ex: core_intents_list")
    module = StringField(required=True, help_text="The path to the module including the class name. Ex: core.intents.Intents")
    function = StringField(required=True, help_text="The function name. Ex: list")
    enabled = BooleanField(default=False, help_text="Boolean to know if the intent is enabled or not")
    meta = {
        'collection': 'intents',
        'allow_inheritance': False
    }

class Rule(DynamicDocument):
    plugin = ReferenceField(Plugin, reverse_delete_rule=CASCADE)
    enabled = BooleanField()
    meta = {
        'collection': 'rules',
        'allow_inheritance': False
    }
class Cron(DynamicDocument):
    plugin = ReferenceField(Plugin, reverse_delete_rule=CASCADE)
    enabled = BooleanField()
    meta = {
        'collection': 'crons',
        'allow_inheritance': False
    }


import json, re, sys, os, inspect, gettext
from pymongo import MongoClient
from twisted.python.reflect import namedAny
from twisted.python import log

class Commands():
    """
        body : contains the text that will be spoken
        type : will be "command"
        from : the message is issued from "LISA Server"
        command : contains the command name
        clients_zone : should be ['sender'] to answer to the client who issued the command,
        * : you can pass other parameters in your json. The client will handle these as he recognize the command name

    """

    def __init__(self, configuration, lisaprotocol):
        self.configuration = configuration
        client = MongoClient(configuration['database']['server'], configuration['database']['port'])
        self.database = client.lisa
        path = os.path.realpath(os.path.abspath(os.path.join(os.path.split(
        inspect.getfile(inspect.currentframe()))[0],os.path.normpath("lang/"))))
        self._ = translation = gettext.translation(domain='lisa', localedir=path, languages=[self.configuration['lang']]).ugettext
        self.lisaprotocol = lisaprotocol

    def mute(self, clientList):
        # ask clients to mute
        # TODO should be fixed, it's not good json to send
        if 'all' in clientList:
            for client in self.lisaprotocol.factory.clients:
                client['object'].sendLine("{'body': 'mute', 'from': 'LISA Server', 'type': 'command'}")
        else:
            for zone in clientList:
                for client in self.lisaprotocol.factory.clients:
                    if client['zone'] == zone:
                            client['object'].sendLine("{'body': 'mute', 'from': 'LISA Server', 'type': 'command'}")

    def parse(self, jsonData):
        if jsonData['body'] == 'LOGIN':
            self.lisaprotocol.answerToClient(json.dumps(
                {
                    'body': self._('The client %(from)s has joined the zone %(zone)s') %
                            {'from': jsonData['from'], 'zone': jsonData['zone']},
                    'command': 'LOGIN',
                    'clients_zone': ['sender'],
                    'from': 'LISA Server',
                    'type': 'command',
                    'bot_name': self.configuration['bot_name']
                }
            ))
        else:
            print jsonData


# Copyright (c) 2008 TechAdept Inc.

'''
'''

from datetime import datetime
from twisted.python import log
from twisted.internet.defer import maybeDeferred

class ScheduledTask(object):
    '''Represents a scheduled task.
    
    :param name: a string with the name of this task
    :param rrule: a dateutil.rrule instance
    :param callable: a callable with the actual work to be done
    :param args: args to send to the callable
    :param kwargs: kwargs to send to the callable
    '''
    def __init__(self, name, rrule, callable, *args, **kwargs):
        self.name = name
        self.callable = callable
        self.recurrence = rrule
        self.next_scheduled_runtime = None
        self.last_runtime = None
        self.running = False

        self.args = args
        self.kwargs = kwargs

        # schedule the task for the first run
        self._reschedule()
        
    def before_execute(self):
        '''Override this method to perform tasks before the callable is executed.
        '''
        log.msg('before_execute called')

    def after_execute(self):
        '''Override this method to perform tasks after the callable is executed.
        '''
        log.msg('after_execute called')

    def _reschedule(self):
        '''Determines the next time that the task should be run.
        '''
        last_time = self.last_runtime
        if last_time is None:
            last_time =  datetime.now()
        next_time = self.recurrence.after(last_time)
        self.next_scheduled_runtime = next_time

    def _post_execute(self, result):
        '''Callback run after the task has finished a run.
        
        Calls the 'after_execute' hook and reschedules the task.
        '''
        self._reschedule()
        self.after_execute()
        return result

    def _execute(self):
        '''Actually runs the task.
        '''
        log.msg('self.args: ')
        log.msg(self.args)
        self.last_runtime = datetime.now()
        self.running = True
        # run the callable
        d = maybeDeferred( self.callable, *self.args, **self.kwargs )
        d.addBoth(self._post_execute)
        return d

    def run(self):
        '''Runs the task.
        
        Client code (usually a ScheduledTaskManager) should call this to 
        get the task to run.
        '''
        self.before_execute()
        d = maybeDeferred( self._execute )
        return d
        
        
        



'''
'''
from twisted.application.internet import TimerService

#~ class SchedulerService(service.Service):
class ScheduledTaskService(TimerService):
    '''Service for running scheduled tasks.
    '''
    def __init__(self, task_manager, interval=1, *args, **kwargs):
        '''
        '''
        self.step = interval
        self.clock = None
        self.call = (task_manager.run, args, kwargs)


    #~ def startService(self):
        #~ pass
        
    #~ def stopService(self):
        #~ pass


import lisa.plugins
import pip
import shutil
import inspect
from lisa.server.web.manageplugins.models import Plugin, Description, Rule, Cron, Intent
import json
import os
from twisted.python.reflect import namedAny
from django.template.loader import render_to_string
import datetime
from lisa.server.web.weblisa.settings import LISA_PATH, configuration
from pymongo import MongoClient
from twisted.python import log
import os

class PluginManager(object):
    """
    """

    def __init__(self):
        self.pkgpath = os.path.dirname(lisa.plugins.__file__)
        self.enabled_plugins = []
        mongo = MongoClient(configuration['database']['server'], configuration['database']['port'])
        self.database = mongo.lisa

    def getEnabledPlugins(self):
        return self.enabled_plugins

    def loadPlugins(self):
        for plugin in self.database.plugins.find( { "enabled": True, "lang": configuration['lang'] } ):
            self.enabled_plugins.append(str(plugin['name']))

        return self.enabled_plugins

    def getPluginByName(self, plugin_name):
        """
        Get the plugin correspoding to a given name
        """
        return None

    def installPlugin(self, plugin_name=None, test_mode=False):
        if Plugin.objects(name=plugin_name):
            return {'status': 'fail', 'log': 'Plugin already installed'}

        if test_mode:
            pip.main(['install', '--quiet', '--install-option=--install-platlib=' + os.getcwd() + '/../',
                      '--install-option=--install-purelib=' + os.getcwd() + '/../', 'lisa-plugin-' + plugin_name])
        else:
            pip.main(['install', 'lisa-plugin-' + plugin_name])
        jsonfile = self.pkgpath + '/' + plugin_name + '/' + plugin_name.lower() + '.json'
        metadata = json.load(open(jsonfile))

        plugin = Plugin()
        description_list = []
        for item in metadata:
            if item != 'crons' and item != 'rules':
                if item == 'description':
                    for description in metadata[item]:
                        oDescription = Description()
                        for k,v in description.iteritems():
                            setattr(oDescription, k, v)
                        description_list.append(oDescription)
                    setattr(plugin, item, description_list)
                elif item == 'enabled':
                    if metadata[item] == 0:
                        setattr(plugin, item, False)
                    else:
                        setattr(plugin, item, True)
                else:
                    setattr(plugin, item, metadata[item])
        plugin.save()

        for item in metadata:
            if item == 'rules':
                for rule_item in metadata['rules']:
                    rule = Rule()
                    for parameter in rule_item:
                        if parameter == 'enabled':
                            if rule_item[parameter] == 0:
                                setattr(rule, parameter, False)
                            else:
                                setattr(rule, parameter, True)
                        else:
                            setattr(rule, parameter, rule_item[parameter])
                    rule.plugin = plugin
                    rule.save()
            if item == 'crons':
                for cron_item in metadata['crons']:
                    cron = Cron()
                    for parameter in cron_item:
                        if parameter == 'enabled':
                            if cron_item[parameter] == 0:
                                setattr(cron, parameter, False)
                            else:
                                setattr(cron, parameter, True)
                        else:
                            setattr(cron, parameter, cron_item[parameter])
                    cron.plugin = plugin
                    cron.save()

        for intent, value in metadata['configuration']['intents'].iteritems():
            oIntent = Intent()
            oIntent.name = intent
            oIntent.function = value['method']
            oIntent.module = '.'.join(['lisa.plugins', plugin_name, 'modules', plugin_name.lower(),
                                       plugin_name])
            oIntent.enabled = True
            oIntent.plugin = plugin
            oIntent.save()
        os.remove(jsonfile)
        return {'status': 'success', 'log': 'Plugin installed'}

    def enablePlugin(self, plugin_name=None, plugin_pk=None):
        if plugin_pk:
            plugin_list = Plugin.objects(pk=plugin_pk)
        else:
            plugin_list = Plugin.objects(name=plugin_name)
        for plugin in plugin_list:
            if plugin.enabled:
                return {'status': 'fail', 'log': 'Plugin already enabled'}
            else:
                plugin.enabled = True
                plugin.save()
                for cron in Cron.objects(plugin=plugin):
                    cron.enabled = True
                    cron.save()
                for rule in Rule.objects(plugin=plugin):
                    rule.enabled = True
                    rule.save()

                intent_list = Intent.objects(plugin=plugin)
                for oIntent in intent_list:
                    oIntent.enabled = True
                    oIntent.save()
                return {'status': 'success', 'log': 'Plugin enabled'}

    def disablePlugin(self, plugin_name=None, plugin_pk=None):
        if plugin_pk:
            plugin_list = Plugin.objects(pk=plugin_pk)
        else:
            plugin_list = Plugin.objects(name=plugin_name)
        for plugin in plugin_list:
            if not plugin.enabled:
                return {'status': 'fail', 'log': 'Plugin already disabled'}
            else:
                plugin.enabled = False
                plugin.save()
                for cron in Cron.objects(plugin=plugin):
                    cron.enabled = False
                    cron.save()
                for rule in Rule.objects(plugin=plugin):
                    rule.enabled = False
                    rule.save()

                intent_list = Intent.objects(plugin=plugin)
                for oIntent in intent_list:
                    oIntent.enabled = False
                    oIntent.save()

                return {'status': 'success', 'log': 'Plugin disabled'}

    def uninstallPlugin(self, plugin_name=None, plugin_pk=None):
        if plugin_pk:
            plugin_list = Plugin.objects(pk=plugin_pk)
        else:
            plugin_list = Plugin.objects(name=plugin_name)
        if not plugin_list:
            return {'status': 'fail', 'log': 'Plugin not installed'}
        else:
            for plugin in plugin_list:
                pip.main(['uninstall', '--quiet', 'lisa-plugin-' + plugin_name])
                plugin.delete()
                for cron in Cron.objects(plugin=plugin):
                    cron.delete()
                for rule in Rule.objects(plugin=plugin):
                    rule.delete()
                intent_list = Intent.objects(plugin=plugin)
                for oIntent in intent_list:
                    oIntent.delete()

            return {'status': 'success', 'log': 'Plugin uninstalled'}

    def methodListPlugin(self, plugin_name=None):
        if plugin_name:
            plugin_list = Plugin.objects(name=plugin_name)
        else:
            plugin_list = Plugin.objects
        listallmethods = []
        for plugin in plugin_list:
            plugininstance = namedAny('.'.join(('lisa.plugins',str(plugin.name), 'modules', str(plugin.name).lower(), str(plugin.name))))()
            listpluginmethods = []
            for m in inspect.getmembers(plugininstance, predicate=inspect.ismethod):
                if not "__init__" in m and not "_" in m:
                    listpluginmethods.append(m[0])
            listallmethods.append({ 'plugin': plugin.name, 'methods': listpluginmethods})
        for f in os.listdir(os.path.normpath(LISA_PATH + '/core')):
            fileName, fileExtension = os.path.splitext(f)
            if os.path.isfile(os.path.join(os.path.normpath(LISA_PATH + '/core'), f)) and not f.startswith('__init__') and fileExtension != '.pyc':
                coreinstance = namedAny('.'.join(('lisa.server.core', str(fileName).lower(), str(fileName).capitalize())))()
                listcoremethods = []
                for m in inspect.getmembers(coreinstance, predicate=inspect.ismethod):
                    #init shouldn't be listed in methods and _ is for translation
                    if not "__init__" in m:
                        listcoremethods.append(m[0])
                listallmethods.append({ 'core': fileName, 'methods': listcoremethods})
        log.msg(listallmethods)
        return listallmethods

    def _template_to_file(self, filename, template, context):
        import codecs
        codecs.open(filename, 'w', 'utf-8').write(render_to_string(template, context))

    def createPlugin(self, plugin_name, author_name, author_email):
        import requests
        import pytz

        metareq = requests.get('/'.join([configuration['plugin_store'], 'plugins.json']))
        if(metareq.ok):
            for item in json.loads(metareq.text or metareq.content):
                if item['name'].lower() == plugin_name.lower():
                    return {'status': 'fail', 'log': 'Plugin already exist in the store'}
        context = {
            'plugin_name': plugin_name,
            'plugin_name_lower': plugin_name.lower(),
            'author_name': author_name,
            'author_email': author_email,
            'creation_date': pytz.UTC.localize(datetime.datetime.now()).strftime("%Y-%m-%d %H:%M%z")
        }
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name))

        # Lang stuff
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/lang'))
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/lang/en'))
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/lang/en/LC_MESSAGES'))
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/lang/en/LC_MESSAGES/' +
                                                    plugin_name.lower() + '.po'),
                          template='plugin/lang/en/LC_MESSAGES/module.po',
                          context=context)

        # Module stuff
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/modules'))
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/modules/' +
                               plugin_name.lower() + '.py'),
                               template='plugin/modules/module.tpl',
                               context=context)
        open(os.path.normpath(self.pkgpath + '/' + plugin_name + '/modules/__init__.py'), "a")

        # Web stuff
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/web'))
        os.mkdir(os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/templates'))
        shutil.copy(src=os.path.normpath(LISA_PATH + '/web/manageplugins/templates/plugin/web/templates/widget.html'),
                    dst=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/templates/widget.html'))
        shutil.copy(src=os.path.normpath(LISA_PATH + '/web/manageplugins/templates/plugin/web/templates/index.html'),
                    dst=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/templates/index.html'))
        open(os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/__init__.py'), "a")
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/api.py'),
                          template='plugin/web/api.tpl',
                          context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/models.py'),
                          template='plugin/web/models.tpl',
                          context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/tests.py'),
                              template='plugin/web/tests.tpl',
                              context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/urls.py'),
                              template='plugin/web/urls.tpl',
                              context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/web/views.py'),
                          template='plugin/web/views.tpl',
                          context=context)

        # Plugin stuff (metadata)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/__init__.py'),
                          template='plugin/__init__.tpl',
                          context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name + '/README.rst'),
                          template='plugin/README.rst',
                          context=context)
        self._template_to_file(filename=os.path.normpath(self.pkgpath + '/' + plugin_name +
                                                    '/' + plugin_name.lower() + '.json'),
                          template='plugin/module.json',
                          context=context)

        return {'status': 'success', 'log': 'Plugin created'}


class PluginManagerSingleton(object):
    """
    Singleton version of the plugin manager.

    Being a singleton, this class should not be initialised explicitly
    and the ``get`` classmethod must be called instead.

    To call one of this class's methods you have to use the ``get``
    method in the following way:
    ``PluginManagerSingleton.get().themethodname(theargs)``
    """

    __instance = None

    def __init__(self):
        """
        Initialisation: this class should not be initialised
        explicitly and the ``get`` classmethod must be called instead.
        """

        if self.__instance is not None:
            raise Exception("Singleton can't be created twice !")

    def get(self):
        """
        Actually create an instance
        """
        if self.__instance is None:
            self.__instance = PluginManager()
            log.msg("PluginManagerSingleton initialised")
        return self.__instance
    get = classmethod(get)


try:
    from web.interface.models.widget import Widget, WidgetUser
    from web.interface.models.workspace import Workspace
except ImportError:
    from lisa.server.web.interface.models.widget import Widget, WidgetUser
    from lisa.server.web.interface.models.workspace import Workspace



from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required

@login_required()
def dashboard(request):
    return render(request, 'dashboard.html', {
    })


def login(request):
    from django.contrib.auth import login
    from mongoengine.django.auth import User
    from mongoengine.queryset import DoesNotExist
    from django.contrib import messages
    if request.method == "POST":
        try:
            user = User.objects.get(username=request.POST['username'])
            if user.check_password(request.POST['password']):
                user.backend = 'mongoengine.django.auth.MongoEngineBackend'
                print login(request, user)
                request.session.set_expiry(60 * 60 * 1)  # 1 hour timeout
                return redirect('dashboard')
            else:
                messages.add_message(request,messages.ERROR, u"Incorrect login name or password !")
        except DoesNotExist:
            messages.add_message(request,messages.ERROR, u"Incorrect login name or password !")
        return render(request, 'login.html', {})
    else:
        return render(request, 'login.html', {})


def logout(request):
    from django.contrib.auth import logout
    logout(request)
    return redirect('login')

@login_required()
def index(request):
    return render(request, 'index.html', {
    })


from django.conf.urls import patterns, url

from lisa.server.web.interface import views

urlpatterns = patterns('',
    url(r'^$', views.dashboard, name='dashboard'),
    url(r'^dashboard$', views.dashboard, name='dashboard'),
    url(r'^login$', views.login, name='login'),
    url(r'^logout$', views.logout, name='logout'),
)



# server/tap.py
from twisted.application import internet, service
from twisted.internet import interfaces
from twisted.python import usage
from lisa.server import service

def makeService(config):
    return service.makeService()



# -*- coding: UTF-8 -*-
import os
import json
import sys
import uuid
from lisa.server.libs import RulesEngine, Commands
from twisted.internet.protocol import Factory
from twisted.protocols.basic import LineReceiver
from twisted.internet import ssl
from twisted.python import log
from OpenSSL import SSL
from lisa.server.libs.txscheduler.manager import ScheduledTaskManager
from lisa.server.libs.txscheduler.service import ScheduledTaskService
from lisa.server.plugins.PluginManager import PluginManagerSingleton

from lisa.server.service import configuration, dir_path
from lisa.server.web.manageplugins.models import Intent, Rule

# Create a task manager to pass it to other services
taskman = ScheduledTaskManager(configuration)
scheduler = ScheduledTaskService(taskman)


class ServerTLSContext(ssl.DefaultOpenSSLContextFactory):
    def __init__(self, *args, **kw):
        kw['sslmethod'] = SSL.TLSv1_METHOD
        ssl.DefaultOpenSSLContextFactory.__init__(self, *args, **kw)


class Lisa(LineReceiver):
    def __init__(self,factory):
        self.factory = factory

    def answerToClient(self, jsondata):
        if configuration['debug']['debug_output']:
            log.msg("OUTPUT: " + str(jsondata))
        jsonreturned = json.loads(jsondata)
        if 'all' in jsonreturned['clients_zone']:
            for client in self.factory.clients:
                client['object'].sendLine(jsondata)
        else:
            for zone in jsonreturned['clients_zone']:
                if zone == 'sender':
                    self.sendLine(jsondata)
                else:
                    for client in self.factory.clients:
                        if client['zone'] == zone:
                            client['object'].sendLine(jsondata)

    def askToClient(self, jsondata):
        jsondata['type'] = 'question'
        self.answerToClient(jsondata=jsondata)

    def connectionMade(self):
        self.client_uuid = str(uuid.uuid1())
        self.factory.clients.append({"object": self, "zone": "", "type": "", "uuid": self.client_uuid})
        if configuration['enable_secure_mode']:
            ctx = ServerTLSContext(
                privateKeyFileName=os.path.normpath(dir_path + '/' + 'configuration/ssl/server.key'),
                certificateFileName= os.path.normpath(dir_path + '/' + 'configuration/ssl/server.crt')
            )
            self.transport.startTLS(ctx, self.factory)
            pass

    def connectionLost(self, reason):
        log.err('Lost connection.  Reason:', reason)
        for client in self.factory.clients:
            if client['object'] == self:
                self.factory.clients.remove(client)

    def lineReceived(self, data):
        if configuration['debug']['debug_input']:
            log.msg("INPUT: " + str(data))
        try:
            jsonData = json.loads(data)
        except ValueError, e:
            self.sendLine("Error : Invalid JSON")
        else:
            for client in self.factory.clients:
                if client['object'] == self and (not client['type'] or not client['zone']):
                    client['type'], client['zone'] = jsonData['type'], jsonData['zone']
            if jsonData['type'] == "chat":
                RulesEngine(configuration).Rules(jsonData=jsonData, lisaprotocol=self)
            elif jsonData['type'] == "command":
                Commands(configuration, lisaprotocol=self).parse(jsonData=jsonData)


class LisaFactory(Factory):
    def __init__(self):
        self.clients = []
        self.syspath = sys.path

    def buildProtocol(self, addr):
        self.Lisa = Lisa(self)
        return self.Lisa

    def LisaReload(self):
        global enabled_plugins
        log.msg("Reloading L.I.S.A Engine")
        sys.path = self.syspath
        enabled_plugins = []
        self.build_activeplugins()

    def SchedReload(self):
        global taskman
        log.msg("Reloading Task Scheduler")
        self.taskman = taskman
        return self.taskman.reload()


class LisaProtocolSingleton(object):
    """
    Singleton version of the Lisa Protocol.

    Being a singleton, this class should not be initialised explicitly
    and the ``get`` classmethod must be called instead.
    """

    __instance = None

    def __init__(self):
        """
        Initialisation: this class should not be initialised
        explicitly and the ``get`` classmethod must be called instead.
        """

        if self.__instance is not None:
            raise Exception("Singleton can't be created twice !")

    def get(self):
        """
        Actually create an instance
        """
        if self.__instance is None:
            self.__instance = Lisa(LisaFactorySingleton.get())
            log.msg("LisaProtocol initialised")
        return self.__instance
    get = classmethod(get)


class LisaFactorySingleton(object):
    """
    Singleton version of the Lisa Factory.

    Being a singleton, this class should not be initialised explicitly
    and the ``get`` classmethod must be called instead.
    """

    __instance = None

    def __init__(self):
        """
        Initialisation: this class should not be initialised
        explicitly and the ``get`` classmethod must be called instead.
        """

        if self.__instance is not None:
            raise Exception("Singleton can't be created twice !")

    def get(self):
        """
        Actually create an instance
        """
        if self.__instance is None:
            self.__instance = LisaFactory()
            log.msg("LisaFactory initialised")
        return self.__instance
    get = classmethod(get)


# Create an instance of factory, then create a protocol instance to import it everywhere
LisaFactorySingleton.get()
LisaProtocolSingleton.get()

# Load the plugins
PluginManagerSingleton.get().loadPlugins()


def Initialize():
    # Create the default core_intents_list intent
    defaults_intent_list = {'name': "core_intents_list",
                     'function': "list",
                     'module': "lisa.server.core.intents.Intents",
                     'enabled': True
    }

    intent_list, created = Intent.objects.get_or_create(name='core_intents_list', defaults=defaults_intent_list)

    # Create the default rule of the rule engine
    defaults_rule = {'name': "DefaultAnwser",
                     'order': 999,
                     'before': None,
                     'after': """lisaprotocol.answerToClient(json.dumps(
                                                {
                                                    'plugin': jsonOutput['plugin'],
                                                    'method': jsonOutput['method'],
                                                    'body': jsonOutput['body'],
                                                    'clients_zone': ['sender'],
                                                    'from': jsonOutput['from']
                                                }))""",
                     'end': True,
                     'enabled': True
    }
    default_rule, created = Rule.objects.get_or_create(name='DefaultAnwser', defaults=defaults_rule)


from django.conf.urls import patterns, include, url
from tastypie.api import Api
from lisa.server.web.manageplugins.api import PluginResource, IntentResource
from lisa.server.web.interface.api import WidgetResource, WorkspaceResource, WidgetByUserResource
from api import LisaResource, UserResource
from twisted.python.reflect import namedAny
import tastypie_swagger

v1_api = Api(api_name='v1')
v1_api.register(UserResource())
v1_api.register(PluginResource())
v1_api.register(IntentResource())
v1_api.register(WorkspaceResource())
v1_api.register(WidgetResource())
v1_api.register(WidgetByUserResource())
v1_api.register(LisaResource())

urlpatterns = patterns('',
    url(r'^speech/', include('lisa.server.web.googlespeech.urls')),
    url(r'^plugins/', include('lisa.server.web.manageplugins.urls')),
    url(r'', include('lisa.server.web.interface.urls')),
)

#Register plugin's API
from lisa.server.plugins.PluginManager import PluginManagerSingleton
for plugin in PluginManagerSingleton.get().getEnabledPlugins():
    urlpatterns += patterns('', url(r'^' + str(plugin) + r'/', include('lisa.plugins.' +
                                                                       str(plugin) + '.web.urls')))
    v1_api.register(namedAny('lisa.plugins.' + plugin + '.web.api.' + plugin + 'Resource')())

urlpatterns += patterns('',
    url(r'^api/', include(v1_api.urls)),
    url(r'^api/docs/', include('tastypie_swagger.urls', namespace='tastypie_swagger')),
)
